//
//  CitiesTableViewController.swift
//  El Vuelo del Grajo
//
//  Created by Santiago Pavón on 7/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import UIKit

class CitiesTableViewController: UITableViewController {

    // Fichero donde salvo los nombres de las ciudades.
    private let SAVED_FILENAME = "cities.plist"
    
    // Las ciudades a mostrar en la tabla.
    var cities: [String]!
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        self.navigationItem.leftBarButtonItem = self.editButtonItem()
        
        loadCities()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return cities.count
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("City Cell", forIndexPath: indexPath) 

        cell.textLabel?.text = cities[indexPath.row]
        
        return cell
    }

    
    override func tableView(tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return "Selecciona una Ciudad"
    }
    
    
    override func tableView(tableView: UITableView, titleForFooterInSection section: Int) -> String? {
        return "IWEB 2015/2016"
    }
    
    
    /*
    // Override to support conditional editing of the table view.
    override func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        // Return NO if you do not want the specified item to be editable.
        return true
    }
    */

    // Override to support editing the table view.
    override func tableView(tableView: UITableView, commitEditingStyle editingStyle: UITableViewCellEditingStyle, forRowAtIndexPath indexPath: NSIndexPath) {
        if editingStyle == .Delete {

            cities.removeAtIndex(indexPath.row)
            
            tableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
            
            saveCities()
            
        } else if editingStyle == .Insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }    
    }

    // Override to support rearranging the table view.
    override func tableView(tableView: UITableView, moveRowAtIndexPath fromIndexPath: NSIndexPath, toIndexPath: NSIndexPath) {

        let aux = cities[fromIndexPath.row]
        
        cities.removeAtIndex(fromIndexPath.row)
        cities.insert(aux, atIndex: toIndexPath.row)
        
        saveCities()
    }

    // Override to support conditional rearranging of the table view.
    override func tableView(tableView: UITableView, canMoveRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }

    
    // MARK: Table View Delegate
    
    override func tableView(tableView: UITableView, titleForDeleteConfirmationButtonForRowAtIndexPath indexPath: NSIndexPath) -> String? {
        return "ñam ñam"
    }
    
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {

        if segue.identifier == "Show City" {
            
            if let cwvc = segue.destinationViewController as? CityWeatherViewController {
                
                if let row = tableView.indexPathForCell(sender as! UITableViewCell)?.row {
                
                    cwvc.city = cities[row]
                }
            }
            
        } else if segue.identifier == "Add City" {
            
            cities.insert("", atIndex: 0)
            
            let ip = NSIndexPath(forRow: 0, inSection: 0)
            
            tableView.insertRowsAtIndexPaths([ip], withRowAnimation: UITableViewRowAnimation.Right)
            
            tableView.selectRowAtIndexPath(ip, animated: true, scrollPosition: .Top)
            
            /*
            tableView.scrollToRowAtIndexPath(ip, atScrollPosition: UITableViewScrollPosition.Top, animated: true)
            */

            if let cevc = segue.destinationViewController as? CityEditorViewController {
                
                cevc.city = ""
            }
            
        } else if segue.identifier == "Edit City" {
            
            if let cevc = segue.destinationViewController as? CityEditorViewController {

                if let ip = tableView.indexPathForCell(sender as! UITableViewCell) {

                    tableView.selectRowAtIndexPath(ip, animated: true, scrollPosition: .Top)
                    
                    cevc.city = cities[ip.row]
                }
            }
        }
    }

    
    @IBAction func cityWasEdited(segue: UIStoryboardSegue) {
                
        if let cevc = segue.sourceViewController as? CityEditorViewController {

            if let ip = tableView.indexPathForSelectedRow {
                
                cities[ip.row] = cevc.city
                
                tableView.reloadRowsAtIndexPaths([ip], withRowAnimation: .Automatic)
                
                saveCities()
            }
            
        }
    }
    
    // MARK: Persistencia
    
    private func getSavedCitiesPath() -> String {
        let path = NSSearchPathForDirectoriesInDomains(.DocumentDirectory,
            .UserDomainMask, true)[0] 
        
        return (path as NSString).stringByAppendingPathComponent(SAVED_FILENAME)
    }
    
    
    private func loadCities() {
        let path = getSavedCitiesPath()
        if let cities = NSArray(contentsOfFile: path) as? [String] {
            self.cities = cities
        } else {
            cities = ["Madrid","Paris","Lisboa","Londres","Moscu",
                "Caracas","Tripoli","Tokio"]
        }
    }
    
    private func saveCities() {
        let path = getSavedCitiesPath()
        (cities as NSArray).writeToFile(path, atomically: true)
    }
}
