//
//  CityWeatherViewController.swift
//  El Vuelo del Grajo
//
//  Created by Santiago Pavón on 7/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import UIKit
import MapKit


class CityWeatherViewController: UITableViewController {

    // URLs Base de la API y de los iconos.
    private let API_OPEN_WEATHER_URL = "http://api.openweathermap.org/data/2.5/weather"

    private let ICON_OPEN_WEATHER_URL = "http://openweathermap.org/img/w"

    // Hay que crearse una cuenta en http://api.openweathermap.org para poder acceder al API.
    private let APIID = ""

    // Ciudad a mostrar
    var city: String!
    
    @IBOutlet weak var temperatureCell: UITableViewCell!
    @IBOutlet weak var temperatureMaxCell: UITableViewCell!
    @IBOutlet weak var temperatureMinCell: UITableViewCell!
    @IBOutlet weak var humidityCell: UITableViewCell!
    @IBOutlet weak var pressureCell: UITableViewCell!
    @IBOutlet weak var sunsetCell: UITableViewCell!
    @IBOutlet weak var sunriseCell: UITableViewCell!
    @IBOutlet weak var mapView: MKMapView!

    
    @IBOutlet var weatherCells: [UITableViewCell]!
    
    
    @IBOutlet weak var w: UILabel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        title = city
    }
    
    
    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        
        checkAPIID()
        downloadWeather()
    }
    
    private func checkAPIID() {
        if APIID == "" {
            let alert = UIAlertController(title: "Regístrese", message: "Para utilizar esta aplicación debe registrarse en http://openweathermap.org y obtener una clase de acceso al API", preferredStyle: .Alert)
            
            alert.addAction(UIAlertAction(
                title: "Vale",
                style: UIAlertActionStyle.Default) {(_: UIAlertAction) -> Void in
                    self.dismissViewControllerAnimated(true, completion: nil)
                })
            
            presentViewController(alert, animated: true, completion: nil)
        }
    }
    
    
    
    // Descargar datos usando NSData(contentsOfURL: url)
    private func downloadWeather() {
        
        UIApplication.sharedApplication().networkActivityIndicatorVisible = true
        defer {
            UIApplication.sharedApplication().networkActivityIndicatorVisible = false
        }
        
        guard let query = "q=\(city)&units=metric&lang=sp&APPID=\(APIID)".stringByAddingPercentEncodingWithAllowedCharacters(.URLQueryAllowedCharacterSet()) else {
            print("Error escapando URL")
            return
        }
        
        guard let url = NSURL(string: "\(API_OPEN_WEATHER_URL)?\(query)") else {
            print("Error: URL mal formada")
            return
        }
        
        guard let data = NSData(contentsOfURL: url) else {
            print("Error: no se han podido descargar los datos.")
            return
        }
        
        do {
            if let dic = try NSJSONSerialization.JSONObjectWithData(data, options: NSJSONReadingOptions()) as? [String:AnyObject] {
                
                print(dic)

                if let cod = dic["cod"] as? Int where cod == 200 {
                    updateOutlets(dic)
                } else {
                    let msg = dic["message"] as? String ?? "Error: Desconocido"
                    print(msg)
                }
            } else {
                print("Los datos recibidos no tienen un formato inesperado.")

            }
        } catch let err  {
            print("No puedo sacar el JSON:", (err as NSError).localizedDescription)
            print(NSString(data: data, encoding: NSUTF8StringEncoding) ?? "Ni idea de lo que esta pasando.")

        }
    }


    private func getIconNamed(name: String) -> UIImage? {
        
        UIApplication.sharedApplication().networkActivityIndicatorVisible = true
        defer {
            UIApplication.sharedApplication().networkActivityIndicatorVisible = false
        }
        
        guard let url = NSURL(string: "\(ICON_OPEN_WEATHER_URL)/\(name).png") else {
            print("Error: URL mal formada")
            return nil
        }
        
        guard let data = NSData(contentsOfURL: url) else {
            print("Error: no se han podido descargar el icono.")
            return nil
        }
        
        guard let img = UIImage(data: data) else {
            print("Los datos recibidos no tienen un formato inesperado.")
            return nil
        }
        
        return img
    }
    
    private func updateOutlets(dic: [String:AnyObject]) {
        
        let main = dic["main"] as! [String:AnyObject]
        let sys = dic["sys"] as! [String:AnyObject]
        let weather = dic["weather"] as! [[String:AnyObject]]
        let coord = dic["coord"] as! [String:AnyObject]
        
        if let latitude = coord["lat"] as? Float {
            if let longitude = coord["lon"] as? Float {
                
                let center = CLLocationCoordinate2D(latitude: CLLocationDegrees(latitude), longitude: CLLocationDegrees(longitude))
                let span = MKCoordinateSpanMake(0.5, 0.5)
                let reg =  MKCoordinateRegionMake(center, span)
                
                mapView.setRegion(reg, animated: false)
                mapView.mapType = .Hybrid
                mapView.userInteractionEnabled = false
            }
        }
        
        if let temp = main["temp"] as? Float {
            temperatureCell.detailTextLabel?.text = "\(temp)ºC"
        }
        if let tempMax = main["temp_max"] as? Float {
            temperatureMaxCell.detailTextLabel?.text = "\(tempMax)ºC"
        }
        if let tempMin = main["temp_min"] as? Float {
            temperatureMinCell.detailTextLabel?.text = "\(tempMin)ºC"
        }
        if let humidity = main["humidity"] as? Float {
            humidityCell.detailTextLabel?.text = "\(humidity)%"
        }
        if let pressure = main["pressure"] as? Float {
            pressureCell.detailTextLabel?.text = "\(pressure)P"
        }
        
        let dateFormatter = NSDateFormatter()
        dateFormatter.dateFormat = "HH:mm"
        
        if let ndate = sys["sunrise"] as? Double {
            let date = NSDate(timeIntervalSince1970: ndate)
            sunriseCell.detailTextLabel?.text = dateFormatter.stringFromDate(date)
        }
        if let ndate = sys["sunset"] as? Double {
            let date = NSDate(timeIntervalSince1970: ndate)
            sunsetCell.detailTextLabel?.text = dateFormatter.stringFromDate(date)
        }

        for i in 0..<5 {
            if weather.count > i {
                if let desc = weather[i]["description"] as? String {
                    weatherCells[i].textLabel?.text = desc
                }
                if let icon = weather[i]["icon"] as? String {
                    weatherCells[i].imageView?.image = getIconNamed(icon)
                }
            } else {
                weatherCells[i].textLabel?.text = ""
                weatherCells[i].imageView?.image = nil
            }
        }
    }


}
